package Heaps;

public class ArrayHeap {

	double[] data;
	int used;
	boolean isMinHeap;
	
	// min heap or max heap
	ArrayHeap (boolean minHeap) {
		data = new double[16];
		used = 0;
		isMinHeap = minHeap;
	}
	
	// the default case: min heap
	ArrayHeap ()
	{
		this(true);
	}
	
	public int NumberElements()
	{
		return used;
	}
	
	// generic comparison: if minheap then return  l<r otherwise l>r
	boolean Compare(double l, double r)
	{
		if (isMinHeap) 
			return l < r; 
		else 
			return l > r; 
	}
	
	// recursive sanity check
	boolean Check(int i)
	{
		boolean check = true;
		if (i*2+1 < used) 
			check = check && Compare(data[i], data[i*2+1]) && Check(i*2+1);
		if (i*2+2 < used) 
			check = check && Compare(data[i], data[i*2+2]) && Check(i*2+2);
		return check;
	}
	
	// binary growth of the array
	void Grow(){
		double[] newData = new double[data.length*2];
		for (int i=0; i<data.length; ++i)
			newData[i] = data[i];
		data = newData;
	}
	
	// insert element to the heap
	public void Insert(double value)
	{
		if (used == data.length) 
			Grow();
		int current = used;
		int parent = (current-1)/2;
		while (current > 0 && Compare(value, data[parent])) {
			data[current] = data[parent];
			current = parent;
			parent = (current-1)/2;
		}
		data[current] = value;
		used++;
		assert(Check(0));
	}
	
	public double GetRoot() {
		assert(used > 0);
		return data[0];
	}
	
	// return child with smaller value. If larger child index (2*current+2) is is not in range, return smaller child index
	private int BestChild(int current)
	{
		if (2*current+2 >= used || Compare(data[2*current+1] , data[2*current+2])) // take left branch 
			return 2*current+1; 
		else // right branch 
			return 2*current+2;
	}
	
	public double ExtractRoot() {
		assert(used > 0);
		double min = data[0];
		double value = data[used-1];
		int current= 0;
		int next = BestChild(current);
		while(next < used && !Compare(value, data[next]))
		{
			data[current] = data[next];
			current=next;
			next = BestChild(current);
		}
		data[current] = value;
		used--;
		assert(Check(0));
		return min;
	}	
	
	void print(){ // for debugging purposes
		for (int i= 0; i<used; ++i)
		{
			System.out.print(data[i]+" ");
		}
		System.out.println();
	}
	
	// heap sort algorithm as a byproduct
	public double[] Sort()
	{
		while (used > 0)
		{
			int pos = used-1;
			data[pos] = ExtractRoot();
		}
		return data;
	}

}
