package ShortestPath;

import java.util.Vector;
import java.util.HashMap;

public class Graph {
	private Vector<Node> nodes;
	private HashMap<String,Node> nodeByName;
	
	Graph(){
		nodes = new Vector<Node>();
		nodeByName = new HashMap<String,Node>();
	}
		
	public Node FindNode(String s){
		return nodeByName.get(s);
	}
	
	public void AddNode(String s){
		Node node = new Node(s);
		nodes.add(node);
		nodeByName.put(s,node);
	}
	
	public void AddEdge(String from, String to, int length)
	{
		Node f = FindNode(from);
		Node t = FindNode(to);
		Edge edge = new Edge(t,length);
		f.AddOutEdge(edge);
	}
	
	Vector<Node> ShortestPath(Node S, Node E)
	{
		Vector<Node> path = new Vector<Node>();
		
		Heap R=new Heap();
		for (int i = 0; i< nodes.size(); ++i)
		{
			Node node = nodes.get(i);
			node.SetPathLen(Integer.MAX_VALUE);
			node.SetPathParent(null);
		}
		S.SetPathLen(0);
		Node newNode = S;
		
		while (newNode != null && newNode != E)
		{
			Vector<Edge> edges = newNode.Edges();
			for (int i = 0; i<edges.size(); ++i)
			{
				Edge edge = edges.get(i);
				int newLength = newNode.GetPathLen() + edge.GetLength();
				Node dest = edge.GetDestination();
				int prevLength = dest.GetPathLen();
				if (newLength < prevLength)
				{
					dest.SetPathLen(newLength);
					dest.SetPathParent(newNode);
					if (prevLength == Integer.MAX_VALUE) // was not in R
						R.Insert(dest);
					else
						R.DecreaseKey(dest);
				}
			}
			newNode = R.ExtractRoot();
		}
		
		while (newNode != null)
		{
			path.add(newNode);
			newNode = newNode.GetPathParent();
		}
		
		return path;
	}
}
